"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""

import time, os.path, string, sys, logfile, gzip, splitFileControler

class InputDataFile:

    def __init__(self, appConfig):

        self.appConfig = appConfig
        self.DATAINPUTPATH = self.appConfig.dataInputFile_Path
        if not os.path.isfile(self.DATAINPUTPATH):
            s = "File not found: %s" %self.DATAINPUTPATH
            sys.exit(s)
        self.DEBUG = 0
        self.BENCHMARK = 0
        self.srcFile = 0
        self.splitFileCntrl = 0

    def openSrcFile(self, mode):
        self.srcFile = open(self.DATAINPUTPATH, mode)

    def closeSrcFile(self):
        self.srcFile.close()   
        
    def display(self, message):
        print time.asctime() + " - " + message        

    def getLstMarkerBasisInfo(self):
        """reads marker information from input file and returns list object of marker dictionaries,
           consisting of {'id': 'xyz', 'ln': 1, 'pvalueRawOrg': 0.2 , 'tvalueOrg': 5.1}
           the permutations are not included in the result set
        """

        markerlist = []
        RAW_P_INCLUDED_TMP = -1
        self.openSrcFile('r')      
        ln = self.srcFile.readline()
        ln = string.strip(ln)
        if not ln: return [], 0
        
        amount_permutations_tmp = -1
        linecount = 0
        while ln:
            marker = {}
            linecount += 1
            ln = string.replace(ln, " ", "\t")
            s = string.split(ln, "\t")

            sCleaned = []
            for part in s:
                if not part: continue
                sCleaned.append(part)

            marker ['ln']         = linecount
            marker ['id']         = sCleaned[0]

            # --------------------------------
            # raw p value
            try:
                if sCleaned[1] == 'NA':
                    RAW_P_INCLUDED = 0
                    marker ['pValRawOrg'] = -99
                elif (float(sCleaned[1])  >= 0) and (float(sCleaned[1]) <= 1):
                    RAW_P_INCLUDED = 1
                    marker ['pValRawOrg'] = float(sCleaned[1])                
                else:
                    sys.exit("Invalid raw p value provided, line " + str(linecount))
            except:
                sys.exit("Invalid raw p value provided, line " + str(linecount))

            if (RAW_P_INCLUDED_TMP <> -1) and (RAW_P_INCLUDED_TMP <> RAW_P_INCLUDED):
                sys.exit("Inconsistent Information for raw p value, line " + str(linecount))                

            RAW_P_INCLUDED_TMP = RAW_P_INCLUDED

            # --------------------------------
            # t value
            try:
                marker ['tValOrg'] = float(sCleaned[2])
            except:                
                sys.exit ("Invalid test statistic in line %i, value %s. Converting to float not possible." %(linecount, str(sCleaned[2])))




            # --------------------------------            
            # check amount permutations per marker
            if self.DEBUG: print "DataInputFile: line %i, checking consistent number of permutations .." %linecount
            amount_permutations = len(sCleaned[3:])
            if amount_permutations_tmp == -1:
                amount_permutations_tmp = amount_permutations
            else:
                if amount_permutations_tmp <> amount_permutations:                    
                    sys.exit("Inconsistent number of given permutation test statistics, compared with previous line; occured in line " + str(linecount))
            if self.DEBUG: print "DataInputFile: line %i, checking consistent number of permutations finished." %linecount
            # --------------------------------
            
            markerlist.append (marker)
            
            ln = self.srcFile.readline()
            ln = string.strip(ln)

        self.closeSrcFile()

        if self.DEBUG:
            lf = logfile.Logfile("meinlog.txt",1)
            for i in markerlist:
                lf.msgEoln(str(i))            
               
        return markerlist, RAW_P_INCLUDED

    def getLstPermutTvalues_useOriginalFile(self, line, markerID):
        """line specifies the line number, starting with line 1,
           for the marker for which the t-values have to be read
        """
        if self.BENCHMARK: self.display('')
        if self.BENCHMARK: self.display('InputFile.getLstPermutTvalues ..')

        linecount = 0
        self.openSrcFile('r')
        if self.BENCHMARK: self.display('Going to line %i ..' %line)
        while linecount < line:
            linecount += 1
            rLn = self.srcFile.readline()
        if self.BENCHMARK: self.display('Going to line %i end' %line)            
        self.closeSrcFile()

        rLn = rLn.strip()             
        rLn = string.replace(rLn, " ", "\t")
        s = string.split(rLn, "\t")

        sCleaned = []
        if self.BENCHMARK: self.display('Cleaning list ..')
        for part in s:
            if not part: continue
            sCleaned.append(part)
        if self.BENCHMARK: self.display('Cleaning list end')

        # security check
        if sCleaned[0] <> markerID: sys.exit("Inconsistency while getting permutation t-vector for marker-id %s" %marker[id])
        
        tVectorPrm = []
        if self.BENCHMARK: self.display('Building final list ..')        
        for i in sCleaned[3:]:
            try:
                tVectorPrm.append(float(i))
            except:
                sys.exit("Invalid test statistic in line %i, value %s. Converting to float not possible." %(line, str(i)))
            
        if self.BENCHMARK: self.display('Building final list end')                    

        if self.BENCHMARK: self.display('InputFile.getLstPermutTvalues end.')
        if self.BENCHMARK: self.display('')
                                    
        return tVectorPrm

    def createSplitFiles (self):
        """creates split files"""
        self.splitFileCntrl = splitFileControler.SplitFileControler(self.appConfig)
        totalLineCount = self.splitFileCntrl.writeSplitFiles()
        return totalLineCount
        
    def getLstPermutTvalues_useSplitFiles(self, line, markerID):
        """line specifies the line number, starting with line 1,
           for the marker for which the t-values have to be read
        """
        if self.BENCHMARK: self.display('Going to line %i ..' %line)
        rLn = self.splitFileCntrl.getLine(line)
        if self.BENCHMARK: self.display('Going to line %i end' %line) 

        rLn = rLn.strip()            
        rLn = string.replace(rLn, " ", "\t")
        s = string.split(rLn, "\t")
        
        sCleaned = []
        if self.BENCHMARK: self.display('Cleaning list ..')
        for part in s:
            if not part: continue
            sCleaned.append(part)
        if self.BENCHMARK: self.display('Cleaning list end')

        # security check
        if sCleaned[0] <> markerID: sys.exit("Inconsistency while getting permutation t-vector for marker-id %s" %marker[id])
        
        tVectorPrm = []
        if self.BENCHMARK: self.display('Building final list ..')        
        for i in sCleaned[3:]:
            try:
                tVectorPrm.append(float(i))
            except:
                sys.exit("Invalid test statistic in line %i, value %s. Converting to float not possible." %(line, str(i)))
        if self.BENCHMARK: self.display('Building final list end')                    

        if self.BENCHMARK: self.display('InputFile.getLstPermutTvalues end.')
        if self.BENCHMARK: self.display('')
                                    
        return tVectorPrm

    def getLstPermutTvalues(self, line, markerID):
        if self.appConfig.useSplitFiles:
            tVectorPrm = self.getLstPermutTvalues_useSplitFiles(line, markerID)             
        else:
            tVectorPrm = self.getLstPermutTvalues_useOriginalFile(line, markerID)
        return tVectorPrm

            
        
        
            

        
        
